@@ -109,10 +109,15 @@ pub(crate) fn extract_function(acc: &mut Assists, ctx: &AssistContext) -> Option
 
             let new_indent = IndentLevel::from_node(&insert_after);
             let old_indent = fun.body.indent_level();
+            let body_contains_await = body_contains_await(&fun.body);
 
-            builder.replace(target_range, format_replacement(ctx, &fun, old_indent));
+            builder.replace(
+                target_range,
+                format_replacement(ctx, &fun, old_indent, body_contains_await),
+            );
 
-            let fn_def = format_function(ctx, module, &fun, old_indent, new_indent);
+            let fn_def =
+                format_function(ctx, module, &fun, old_indent, new_indent, body_contains_await);
             let insert_offset = insert_after.text_range().end();
             match ctx.config.snippet_cap {
                 Some(cap) => builder.insert_snippet(cap, insert_offset, fn_def),
@@ -336,9 +341,9 @@ impl Param {
         let var = self.var.name(ctx.db()).unwrap().to_string();
         let var_name = make::name(&var);
         let pat = match self.kind() {
-            ParamKind::MutValue => make::ident_mut_pat(var_name),
+            ParamKind::MutValue => make::ident_pat(false, true, var_name),
             ParamKind::Value | ParamKind::SharedRef | ParamKind::MutRef => {
-                make::ident_pat(var_name)
+                make::ext::simple_ident_pat(var_name)
             }
         };
 
@@ -954,7 +959,12 @@ fn scope_for_fn_insertion_node(node: &SyntaxNode, anchor: Anchor) -> Option<Synt
     last_ancestor
 }
 
-fn format_replacement(ctx: &AssistContext, fun: &Function, indent: IndentLevel) -> String {
+fn format_replacement(
+    ctx: &AssistContext,
+    fun: &Function,
+    indent: IndentLevel,
+    body_contains_await: bool,
+) -> String {
     let ret_ty = fun.return_type(ctx);
 
     let args = fun.params.iter().map(|param| param.to_arg(ctx));
@@ -994,6 +1004,9 @@ fn format_replacement(ctx: &AssistContext, fun: &Function, indent: IndentLevel)
         }
     }
     format_to!(buf, "{}", expr);
+    if body_contains_await {
+        buf.push_str(".await");
+    }
     if fun.ret_ty.is_unit()
         && (!fun.vars_defined_in_body_and_outlive.is_empty() || !expr.is_block_like())
     {
@@ -1059,7 +1072,7 @@ impl FlowHandler {
             }
             FlowHandler::IfOption { action } => {
                 let path = make::ext::ident_path("Some");
-                let value_pat = make::ident_pat(make::name("value"));
+                let value_pat = make::ext::simple_ident_pat(make::name("value"));
                 let pattern = make::tuple_struct_pat(path, iter::once(value_pat.into()));
                 let cond = make::condition(call_expr, Some(pattern.into()));
                 let value = make::expr_path(make::ext::ident_path("value"));
@@ -1073,15 +1086,15 @@ impl FlowHandler {
 
                 let some_arm = {
                     let path = make::ext::ident_path("Some");
-                    let value_pat = make::ident_pat(make::name(some_name));
+                    let value_pat = make::ext::simple_ident_pat(make::name(some_name));
                     let pat = make::tuple_struct_pat(path, iter::once(value_pat.into()));
                     let value = make::expr_path(make::ext::ident_path(some_name));
-                    make::match_arm(iter::once(pat.into()), value)
+                    make::match_arm(iter::once(pat.into()), None, value)
                 };
                 let none_arm = {
                     let path = make::ext::ident_path("None");
                     let pat = make::path_pat(path);
-                    make::match_arm(iter::once(pat), none.make_result_handler(None))
+                    make::match_arm(iter::once(pat), None, none.make_result_handler(None))
                 };
                 let arms = make::match_arm_list(vec![some_arm, none_arm]);
                 make::expr_match(call_expr, arms)
@@ -1092,17 +1105,21 @@ impl FlowHandler {
 
                 let ok_arm = {
                     let path = make::ext::ident_path("Ok");
-                    let value_pat = make::ident_pat(make::name(ok_name));
+                    let value_pat = make::ext::simple_ident_pat(make::name(ok_name));
                     let pat = make::tuple_struct_pat(path, iter::once(value_pat.into()));
                     let value = make::expr_path(make::ext::ident_path(ok_name));
-                    make::match_arm(iter::once(pat.into()), value)
+                    make::match_arm(iter::once(pat.into()), None, value)
                 };
                 let err_arm = {
                     let path = make::ext::ident_path("Err");
-                    let value_pat = make::ident_pat(make::name(err_name));
+                    let value_pat = make::ext::simple_ident_pat(make::name(err_name));
                     let pat = make::tuple_struct_pat(path, iter::once(value_pat.into()));
                     let value = make::expr_path(make::ext::ident_path(err_name));
-                    make::match_arm(iter::once(pat.into()), err.make_result_handler(Some(value)))
+                    make::match_arm(
+                        iter::once(pat.into()),
+                        None,
+                        err.make_result_handler(Some(value)),
+                    )
                 };
                 let arms = make::match_arm_list(vec![ok_arm, err_arm]);
                 make::expr_match(call_expr, arms)
@@ -1122,12 +1139,13 @@ fn format_function(
     fun: &Function,
     old_indent: IndentLevel,
     new_indent: IndentLevel,
+    body_contains_await: bool,
 ) -> String {
     let mut fn_def = String::new();
     let params = make_param_list(ctx, module, fun);
     let ret_ty = make_ret_ty(ctx, module, fun);
     let body = make_body(ctx, old_indent, new_indent, fun);
-    let async_kw = if body_contains_await(&fun.body) { "async " } else { "" };
+    let async_kw = if body_contains_await { "async " } else { "" };
     match ctx.config.snippet_cap {
         Some(_) => format_to!(fn_def, "\n\n{}{}fn $0{}{}", new_indent, async_kw, fun.name, params),
         None => format_to!(fn_def, "\n\n{}{}fn {}{}", new_indent, async_kw, fun.name, params),
@@ -3681,7 +3699,7 @@ async fn some_function() {
 "#,
             r#"
 fn main() {
-    fun_name();
+    fun_name().await;
 }
 
 async fn $0fun_name() {
@@ -3710,7 +3728,7 @@ async fn some_function() {
 "#,
             r#"
 fn main() {
-    fun_name();
+    fun_name().await;
 }
 
 async fn $0fun_name() {
